#
#               Common Quantification
#                        of
#               A Categorical Variable
#
#              Yasuharu Okamoto, 2016.09
#
#
#

import numpy
import numpy as np
from numpy import linalg
import math
from eigen4me import eigen_sym as eigen

#
#       X:      The categorical variable to bu quantified
#       NCat:   The number of categories in the variable
#       NCase:  The number of cases
#

def comq( X, NCat, NCase ):
        G = [0]*NCase                   #  A list for the indicator matrix
        for i in range(0, NCase):
                if (X[i] < 1) or (X[i] > NCat):
                         print("Category is out of the range...X[%d] = %d"%(i,X[i]))
                         raise "CatRange"
                        
                G[i] = []
                for j in range(0, NCat):
                    if (X[i] == j + 1):
                        G[i].append(1)
                    else:
                        G[i].append(0)
 
        G = np.array(G)

        onesRow = np.full((1, NCase), 1)

        onesCol = np.full((NCase, 1), 1)

        f = onesRow @ G
        H = G - (onesCol @ f) / NCase
        HpH = H.transpose() @ H

        w, v = eigen(HpH)               #  Eigen decomposition of HpH = H'H

        n_quant = 0;                    #  n_quant: the number of non-zero eigen values
        while (w[n_quant] > 1.0e-7):
            n_quant = n_quant + 1

        eigen_val = w[0: n_quant]
        eigen_vec = v[0: n_quant]

        matOmega = eigen_vec.transpose()

#
#       Calculation of the standardized scores of quantification
#

        z = H @ matOmega

        invLmbdN = np.full((n_quant, n_quant), 0.0)
        for i in range(n_quant):
                invLmbdN[i, i] = 1.0 / ((eigen_val[i] / NCase) ** 0.5)

        z = z @ invLmbdN

        
                                        
        return n_quant, eigen_val, matOmega, z          #       n_quant:        The number of quantification variables
                                                        #       eigen_val:      The vector of eigen values of quantification
                                                        #       matOmega:       The matrix of the eigen vectpors (the weights)
                                                        #                       of quantification

